# -*- coding: utf-8 -*
from sklearn import linear_model


class Stats():

    def lreg(self, xcol, ycol, name="regression"):
        """
        Add a column to the main dataframe populted with
        the model's linear regression for a column
        """
        try:
            x = self.df[xcol].values.reshape(-1, 1)
            y = self.df[ycol]
            lm = linear_model.LinearRegression()
            lm.fit(x, y)
            predictions = lm.predict(x)
            self.df[name] = predictions
        except Exception as e:
            self.err(e, self.lreg, "Can not calculate linear regression")



import pandas as pd
from numpy.core.numeric import nan
from .db import Db
from .charts import Plot
from .data import Df
from .report import Report
from .errors import Errors
from .messages import Messages

__version__ = "0.4.13"


class DataSwim(Db, Df, Plot, Report, Errors, Messages):
    """
    Main class
    """

    def __init__(self, df=None, db=None):
        """
        Initialize with an empty dataframe
        """
        global __version__
        self.version = __version__
        self.df = df
        self.db = db
        self.x = None
        self.y = None
        self.chart_obj = None
        self.chart_opts = dict(width=940)
        self.chart_style = {}
        self.engine = "bokeh"
        self.label = None
        self.backup_df = None
        self.autoprint = True
        self.errors_handling = "exceptions"
        self.notebook = False
        self.header = self._header
        self.footer = self._footer
        self.reports = []
        self.report_engines = [self.engine]
        self.start_time = None
        self.influx_cli = None
        self.datapath = None
        self.report_path = None
        self.static_path = None
        self.quiet = False
        self.nan = nan

    def resetall(self):
        self.__init__(self.df, self.db)

    def __repr__(self):
        num = 0
        if self.df is not None:
            num = len(self.df.index)
        msg = "<DataSwim object | " + str(num) + " rows>"
        return msg

    def new_(self, df=pd.DataFrame(), db=None, quiet=False):
        """
        Returns a new DataSwim instance from a dataframe
        """
        try:
            ds2 = DataSwim(df, db)
        except Exception as e:
            self.err(e, self.new_, "Can not set new instance")
        if self.autoprint is True and quiet is False:
            self.ok("A new instance was created")
        return ds2

    def set_df(self, data, **args):
        """
        Set a dataframe and an instance
        """
        self.df = pd.DataFrame(data, **args)


ds = DataSwim()



# -*- coding: utf-8 -*-

from __future__ import print_function
from goerr import err


class Errors():
    """
    Class to handle errors
    """

    def __init__(self, df=None, db=None):
        """
        Initialize with an empty dataframe
        """
        self.autoprint = True
        self.errors_handling = "exceptions"

    def err(self, *args):
        """
        Error handling
        """
        ex = None
        msg = None
        func = None
        for arg in args:
            if isinstance(arg, Exception):
                ex = arg
            elif isinstance(arg, str):
                msg = arg
            elif callable(arg) is True:
                func = arg
        if ex is not None:
            err.new(ex)
            if len(args) < 3:
                return
        if func is None or msg is None:
            f = ""
            if func is not None:
                f = "(from function " + str(func) + ")"
            err.new(
                "Please provide a function and a message to the error constructor " + f)
        err.new(msg, func)
        if self.errors_handling == "trace":
            self.debug(str(len(err.errs)) + ".",
                       "An error has occured: use trace() to get the stack trace")
        else:
            self.trace()

    def trace(self):
        """
        Prints the error trace
        """
        if err.exists:
            print("")
            err.throw(reverse=True)

    def errs(self):
        """
        Sets the error handling mode to trace
        """
        self.errors_handling = "trace"



import os
from IPython.core.display import display, HTML


class Report():
    """
    Class to handle reporting
    """

    def __init__(self, df=None):
        """
        Initialize
        """
        self.engine = "bokeh"
        self.header = self._header
        self.footer = self._footer
        self.reports = []
        self.report_path = None
        self.report_engines = [self.engine]

    def stack(self, slug, chart_obj=None, title=None):
        """
        Get the html for a chart and store it
        """
        if chart_obj is None:
            if self.chart_obj is None:
                self.err(
                    self.stack,
                    "No chart object set: please provide one in parameters")
                return
            chart_obj = self.chart_obj
        try:
            seaborn_chart = None
            if self.engine == "chartjs":
                html = chart_obj
            elif self.engine == "seaborn":
                html = ""
                seaborn_chart = chart_obj
            else:
                html = self.get_html(chart_obj, slug)
            if html is None and seaborn_chart is None:
                self.err(
                    self.stack, "Can not stack: empty html reveived for " +
                    str(chart_obj), "-", slug)
                return
            report = dict(slug=slug, html=html)
            if seaborn_chart is not None:
                report["seaborn_chart"] = seaborn_chart
            if self.engine not in self.report_engines:
                self.report_engines.append(self.engine)
            self.reports.append(report)
        except Exception as e:
            self.err(e, self.stack, "Can not stack report")
            return
        if self.autoprint is True:
            self.ok("Stacked report", slug)

    def to_file(self, slug, folderpath=None, header=None, footer=None):
        """
        Writes the html report to a file from the report stack
        """
        if folderpath is None:
            if self.report_path is None:
                self.err(
                    self.to_file,
                    "Please set the report_path parameter or pass a path in arguments")
                return
            folderpath = self.report_path
        else:
            self.report_path = folderpath
        html = self._get_header(header)
        if html is None or html == "":
            self.err(self.to_file, "Can not get html header")
        for report in self.reports:
            html += report["html"]
        html += self._get_footer(footer)
        try:
            path = self._write_file(slug, folderpath, html)
            path = "file://" + path
        except Exception as e:
            self.err(e, self.to_file, "Can not save report to file")
            return
        self.reports = self.report_engines = []
        if self.autoprint is True:
            link = '<a href="' + path + '" target="_blank">' + path + '</a>'
            return display(HTML(link))
        else:
            return None

    def to_files(self, folderpath=None):
        """
        Writes the html report to one file per report
        """
        fp = folderpath
        if folderpath is None:
            if self.report_path is None and "seaborn" \
                    not in self.report_engines:
                self.err(self.to_files,
                         "No folder path set for reports: please provide "
                         "one as argument or set ds.report_path")
                return
            folderpath = self.report_path
        else:
            if folderpath.startswith('/') or folderpath.startswith("."):
                pass
            else:
                folderpath = self.report_path + "/" + folderpath
        try:
            for report in self.reports:
                if "html" not in report:
                    self.err(self.to_files,
                             "No html for report " + report)
                    self.reports = self.report_engines = []
                    return
                if "seaborn_chart" in report:
                    if fp is None and self.static_path is None:
                        self.err("Please provide a folder path or set the "
                                 "static_path property")
                        return
                    if self.static_path is None:
                        folderpath = fp
                    else:
                        folderpath = self.static_path + "/" + fp
                    self._save_seaborn_chart(report, folderpath)
                else:
                    html = report["html"]
                    self._write_file(report["slug"], folderpath, html)
            self.reports = []
            self.report_engines = []
        except Exception as e:
            self.err(e, self.to_files, "Can not save reports to files")
            return
        if self.autoprint is True:
            self.ok("Data writen to files")

    def get_html(self, chart_obj=None, slug=None):
        """
        Get the html and script tag for a chart
        """
        if chart_obj is None:
            if self.chart_obj is None:
                self.err(
                    self.get_html,
                    "No chart object registered, please provide "
                    "one in parameters"
                )
                return
            chart_obj = self.chart_obj
        try:
            if self.engine == "bokeh":
                html = self._get_bokeh_html(chart_obj)
                if html is None:
                    self.err(self.get_html,
                             "No html returned for " + str(chart_obj))
                return html
            elif self.engine == "altair":
                html = self._get_altair_html(chart_obj, slug)
                if html is None:
                    self.err(self.get_html,
                             "No html returned for " + str(chart_obj))
                return html
            else:
                self.err(self.get_html, "Chart engine " +
                         self.engine + " unknown")
                return
        except Exception as e:
            self.err(e, self.get_html, "Can not get html from chart object")

    def set_engine(self, engine):
        """
        Sets a chart engine reseting all the others in the report stack
        """
        self.engine = engine
        self.report_engines = [engine]

    def _get_header(self, header):
        """
        Gets the html header
        """
        if header is None:
            html = self.header()
        else:
            html = header
        return html

    def _get_footer(self, footer):
        """
        Gets the html footer
        """
        if footer is None:
            html = self.footer()
        else:
            html = footer
        return html

    def _write_file(self, slug, folderpath, html):
        """
        Writes a chart's html to a file
        """
        # check directories
        if not os.path.isdir(folderpath):
            try:
                os.makedirs(folderpath)
            except Exception as e:
                self.err(e)
                return
        # construct file path
        filepath = folderpath + "/" + slug + ".html"
        # write the file
        try:
            filex = open(filepath, "w")
            filex.write(html)
            filex.close()
            if self.autoprint is True:
                if self.notebook is False:
                    self.ok("File written to", filepath)
                else:
                    html = '<a href="' + filepath + '">' + filepath + '</a>'
                    self.html("File written to", html)
        except Exception as e:
            self.err(e)
        return filepath

    def _header(self):
        """
        Default html header
        """
        html = """
		<!DOCTYPE html>
		<html lang="en">
		<head>
			<meta charset="utf-8">
			<title>Report</title>
		"""
        if "bokeh" in self.report_engines:
            html += self.bokeh_header_()
        if "altair" in self.report_engines:
            html += self.altair_header_()
        if "chartjs" in self.report_engines:
            html += self.chartjs_header_()
        html += """
		</head>
		<body>
		"""
        return html

    def _footer(self):
        """
        Default html footer
        """
        return "</body>\n</html>"



from numpy.core.numeric import nan


class Relation():
    """
    A class to handle database relations
    """

    def __init__(self, db=None):
        """
        Initialize with an empty db
        """
        self.db = db

    def relation(self, search_ds, origin_field, search_field, destination_field=None,
                 id_field="id"):
        """
        Add a column to the main dataframe from a relation foreign key 
        """
        df = self._relation(search_ds, origin_field,
                            search_field, destination_field, id_field)
        self.df = df

    def relation_(self, search_ds, origin_field, search_field, destination_field=None,
                  id_field="id"):
        """
        Returns a DataSwim instance with a column filled from a relation foreign key 
        """
        df = self._relation(search_ds, origin_field,
                            search_field, destination_field, id_field)
        return self.clone_(df)

    def _relation(self, search_ds, origin_field, search_field, destination_field, id_field):
        """
        Add a column to the main dataframe from a relation foreign key 
        """
        if self.autoprint is True:
            self.start("Processing relation",
                       origin_field, "->", search_field)
        df = self.df.copy()
        if destination_field is None:
            destination_field = search_field
        df[destination_field] = None

        if origin_field not in self.df.columns.values:
            msg = "Can not find field " + origin_field + " in dataset: "
            msg += "found fields " + ", ".join(list(self.df.columns.values))
            self.warning(msg)
            return
        if search_field not in search_ds.df.columns.values:
            msg = "Can not find field " + search_field + " in related dataset: "
            msg += "found fields " + \
                ", ".join(list(search_ds.df.columns.values))
            self.warning(msg)
            return

        def set_rel(row):
            try:
                origin_val = row[origin_field]
                end_val = search_ds.df.loc[search_ds.df[id_field]
                                           == origin_val][search_field].values[0]
                return end_val
            except:
                return nan

        try:
            df = df.copy()
            df[destination_field] = df.apply(set_rel, axis=1)
        except Exception as e:
            self.err(e, self._relation, "Can not get relation data")
            return
        if self.autoprint is True:
            self.end("Finished processing relation")
        return df



import uuid
from chartjspy import chart


class Chartjs():
    """
    A class to handle charts with the Chartjs library
    """

    def radar_(self, label=None, style=None, opts=None, options={}):
        """
        Get a radar chart
        """
        try:
            return self._get_chart("radar", style=style, opts=opts, label=label, options=options)
        except Exception as e:
            self.err(e, self.radar_, "Can not draw radar chart")

    def _get_chartjs_chart(self, xcol, ycol, chart_type, label=None, opts={},
                           style={}, options={}, **kwargs):
        """
        Get Chartjs html
        """
        try:
            xdata = list(self.df[xcol])
        except Exception as e:
            self.err(e, self._get_chartjs_chart,
                     "Can not get data for x field ", ycol)
            return
        if label is None:
            label = "Data"
        try:
            if type(ycol) != list:
                ydata = [dict(name=label, data=list(self.df[ycol]))]
            else:
                ydata = []
                for col in ycol:
                    y = {}
                    y["name"] = col
                    y["data"] = list(self.df[col])
                    ydata.append(y)
        except Exception as e:
            self.err(e, self._get_chartjs_chart,
                     "Can not get data for y field ", xcol)
            return
        try:
            slug = str(uuid.uuid4())
            html = chart.get(slug, xdata, ydata, label,
                             opts, style, chart_type, **kwargs)
            return html
        except Exception as e:
            self.err(e, self._get_chartjs_chart, "Can not get chart")

    def chartjs_header_(self):
        """
        Returns html script tags for Chartjs
        """
        header = """
		<script type="text/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.7.1/Chart.bundle.min.js"></script>
		"""
        return header



from setuptools import setup, find_packages


version = __import__('dataswim').__version__

setup(
    name='dataswim',
    packages=find_packages(),
    version=version,
    description='Utilities to swim in a data lake',
    author='synw',
    author_email='synwe@yahoo.com',
    url='https://github.com/synw/dataswim',
    download_url='https://github.com/synw/dataswim/releases/tag/' + version,
    keywords=['data_visualization', "data_exploration", "charts"],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
    install_requires=[
        'pandas',
        'scikit-learn',
        'dataset',
        "pytablewriter",
        'goerr',
        'holoviews==1.9.1',
        'bokeh==0.12.11',
        'altair==1.2.1',
        'gencharts',
        'influxdb',
        'chartjspy',
        "seaborn",
        "arrow",
    ],
    zip_safe=False
)
